#!/usr/bin/env python
"""
Copyright (c) 2011, matthewharrison@gmail.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
    Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


TODO:

* re-number footnotes with each chapter
* Dropcap cmd line option
* smartypants option
* Populate metadata from rst
* Cover generation
 * see http://blog.threepress.org/2009/11/20/best-practices-in-epub-cover-images/
 * Should probably convert pngs to jpegs
* Check xhtml validation

"""
from contextlib import contextmanager
import os
import sys


from genshi.util import striptags
import docutils
from docutils.core import Publisher, default_description, \
    default_usage
from docutils import io, nodes
from docutils.parsers.rst import Directive, directives
from docutils.readers import standalone
from docutils.writers import html4css1

try:
    import smartypants
except:
    smartypants = None

from epublib import epub


@contextmanager
def cwd_cm():
    cur_dir = os.getcwd()
    found = False
    for arg in sys.argv:
        if arg.endswith(".rst"):
            found = arg
            break
    if found:
        directory = os.path.dirname(arg)
        if directory:
            os.chdir(directory)
    yield
    os.chdir(cur_dir)


def cwd_decorator(func):
    """
    decorator to change cwd to directory containing rst for this function
    """
    def wrapper(*args, **kw):
        cur_dir = os.getcwd()
        found = False
        for arg in sys.argv:
            if arg.endswith(".rst"):
                found = arg
                break
        if found:
            directory = os.path.dirname(arg)
            if directory:
                os.chdir(directory)
        data = func(*args, **kw)
        os.chdir(cur_dir)
        return data
    return wrapper


class EpubWriter(html4css1.Writer):
    def __init__(self):
        html4css1.Writer.__init__(self)
        self.translator_class = HTMLTranslator

    def translate(self):
        self.visitor = visitor = self.translator_class(self.document)
        self.document.walkabout(visitor)
        for attr in self.visitor_attributes:
            setattr(self, attr, getattr(visitor, attr))
        self.output = self.visitor.get_output()


class HTMLTranslator(html4css1.HTMLTranslator):
    def __init__(self, document):
        html4css1.HTMLTranslator.__init__(self, document)
        self.book = epub.EpubBook()
        self.sections = []
        self.body_len_before_node = {}
        self.section_title = ''
        self.authors = []
        self.cover_image = None
        self._ignore_image = False
        self.images = {}  # absolute path to book path
        self.first_page = True
        self.field_name = None
        self.fields = {}
        self.in_node = {}
        self.is_title_page = False
        self.first_paragraph = True
        self.css = ['main.css']
        self.js = []
        self.font = []  # user embedded font paths
        self.toc_parents = []
        self.toc_page = False
        self.toc_entry = True  # page that has entry in TOC and NCX
        self.parent_level = 0
        self.guide_type=None

    def dispatch_visit(self, node):
        # mark body length before visiting node
        self.body_len_before_node[node.__class__.__name__] = len(self.body)
        # keep track of parents
        count = self.in_node.setdefault(node.tagname, 0)
        self.in_node[node.tagname] += 1
        html4css1.HTMLTranslator.dispatch_visit(self, node)

    def dispatch_departure(self, node):
        try:
            self.in_node[node.tagname] -= 1
        except KeyError as e:
            print node.tagname
        html4css1.HTMLTranslator.dispatch_departure(self, node)

    def at(self, nodename):
        """
        shortcut for at/under this node
        """
        return self.in_node.get(nodename, False)

    def _dumb(self, node):
        pass
    visit_comment = _dumb
    depart_comment = _dumb

    def visit_envvar(self, node):
        # !!! hook up to index when motivated
        self.visit_literal(node)

    def depart_envvar(self, node):
        self.depart_literal(node)

    def visit_paragraph(self, node):
        # All text need p's (else breaks epubcheck)
        if self.first_paragraph and not self.at('admonition'):
            if self.section_level == 1:
                self.body.append(self.starttag(node, 'p', '', **{'class':'first-para-chapter'}))
            else:
                self.body.append(self.starttag(node, 'p', '', **{'class':'first-para'}))
            self.context.append('</p>\n')

        elif self.at('admonition'):
            if self.first_paragraph:
                self.body.append(self.starttag(node, 'p', '', **{'class':'note-first-p'}))
            else:
                self.body.append(self.starttag(node, 'p', '', **{'class':'note-p'}))
            self.context.append('</p>\n')
        elif self.at('block_quote'):
            self.body.append(self.starttag(node, 'p', '', **{}))
            self.context.append('</p>\n')

        else:
            html4css1.HTMLTranslator.visit_paragraph(self, node)
        self.first_paragraph = False

    def append_class_on_child(self, node, class_, index=0):
        children = [n for n in node if not isinstance(n, nodes.Invisible)]
        try:
            child = children[index]
        except IndexError:
            return
        # should only be one class
        if child['classes']:
            child['classes'] = child['classes'][0] + class_

    def set_first_last(self, node):
        # mobi doesn't like multiple classes per tag
        self.append_class_on_child(node, '-first', 0)

    def visit_literal_block(self, node):
        # mobi needs an extra div here, otherwise headings following <pre> are indented poorly
        if self.at('admonition'):
            self.body.append(self.starttag(node, 'div', CLASS='div-literal-block-admonition'))
            self.body.append(self.starttag(node, 'pre', CLASS='literal-block-admonition'))
        else:
            self.body.append(self.starttag(node, 'div'))
            self.body.append(self.starttag(node, 'pre', CLASS='literal-block'))

    def depart_literal_block(self, node):
        html4css1.HTMLTranslator.depart_literal_block(self, node)
        self.body.append('</div>\n')

    def visit_meta(self, node):
        # support gutenberg extensions
        # http://www.gutenberg.org/wiki/Gutenberg:The_PG_boilerplate_for_RST
        name = node.get('name', '')
        if name.startswith('DC'):
            self.fields[name[3:]] = node.get('content')
        elif name.startswith('coverpage'):
            with cwd_cm():
                cover_page = node.get('content')
                self.cover_image = os.path.abspath(cover_page)

    @cwd_decorator
    def visit_Text(self, node):
        if "Copyright" in str(node):
            print self.body
        txt = node.astext()
        if self.at('field_name'):
            self.field_name = node.astext()
        elif self.at('field_body'):
            pass
        elif self.at('generated'):#  and 'sectnum' in node.get('classes'):
            # avoid <generated classes="sectnum">5.1</generated>
            pass
        elif self.at('comment'):
            if txt == 'titlepage':
                self.is_title_page = True
            elif txt.startswith('newpage:'):
                # don't create table of contents entry
                self.create_chapter()
                self.toc_entry = False
                self.section_level = 1
            elif txt.startswith('newchapter:'):
                # don't create table of contents entry
                self.create_chapter()
                self.section_level = 1
            elif txt.startswith('guide:'):
                self.guide_type=txt.split(':')[-1]
            elif txt.startswith('css:'):
                paths = txt.split(':')[-1].split(',')
                self.css = self.css + [os.path.abspath(path) for path in paths if path]
            elif txt.startswith('js:'):
                paths = txt.split(':')[-1].split(',')
                self.js = self.js + [os.path.abspath(path) for path in paths if path]
            elif txt.startswith('font:'):
                paths = txt.split(':')[-1].split(',')
                self.font = self.font + [os.path.abspath(path) for path in paths if path]
            elif txt == 'nocss':
                self.css = None
            elif txt.startswith('addimg:'):
                uri = txt.split(':')[-1]
                self.images[os.path.abspath(uri)] = uri
            elif txt.startswith('toc:show'):
                # old school hack now overriding .. contents::
                # see visit_epubcontent
                if self.body:
                    self.create_chapter()
                self.book.add_toc_page(order=self.book.next_order())
            elif txt.startswith('toc:parent1'):
                self.is_parent = True
                self.parent_level = 1
                self.toc_parents = []
            elif txt.startswith('toc:parent2'):
                self.is_parent = True
                self.parent_level = 2
            elif txt.startswith('toc:clear'):
                self.is_parent = False
                self.toc_parents = []
        else:
            html4css1.HTMLTranslator.visit_Text(self, node)

    def visit_epubcontent(self, node):
        if self.body:
            print "CREATING BEFORE"
            # create an existing chapter
            self.create_chapter()
        self.toc_page = True

    def depart_epubcontent(self, node):
        self.section_level = 0

    @cwd_decorator
    def visit_image(self, node):
        self._ignore_image = False
        if 'cover' in node.get('classes'):
            source = node.get('uri')
            self.cover_image = os.path.abspath(source)
            self._ignore_image = True
        else:
            source = node.get('uri')
            self.images[os.path.abspath(source)] = source
        if not self._ignore_image:
            # appease epubcheck
            self.body.append('<div>\n')
            html4css1.HTMLTranslator.visit_image(self, node)

    def depart_image(self, node):
        if not self._ignore_image:
            # appease epubcheck
            self.body.append('</div>\n')
            html4css1.HTMLTranslator.depart_image(self, node)

    def visit_field_list(self, node):
        if self.first_page:
            pass
        else:
            return html4css1.HTMLTranslator.visit_field_list(self, node)

    def depart_field_list(self, node):
        if self.first_page:
            pass
        else:
            return html4css1.HTMLTranslator.depart_field_list(self, node)

    def visit_field(self, node):
        if self.first_page:
            pass
        else:
            return html4css1.HTMLTranslator.visit_field(self, node)

    def depart_field(self, node):
        if self.first_page:
            pass
        else:
            return html4css1.HTMLTranslator.depart_field(self, node)

    def visit_field_body(self, node):
        if self.first_page:
            #pass
            # need to deal with para's that have multiple text children
            # such as foo... |copy| foo :(
            self.fields[self.field_name] = node.astext()
        else:
            return html4css1.HTMLTranslator.visit_field_body(self, node)

    def depart_field_body(self, node):
        if self.first_page:
            pass
        else:
            return html4css1.HTMLTranslator.depart_field_body(self, node)

    def visit_field_name(self, node):
        if self.first_page:
            pass
        else:
            return html4css1.HTMLTranslator.visit_field_name(self, node)

    def depart_field_name(self, node):
        if self.first_page:
            pass
        else:
            return html4css1.HTMLTranslator.depart_field_name(self, node)

    def visit_title(self, node):
        print "TITLE", self.section_level, node
        html4css1.HTMLTranslator.visit_title(self, node)

    def depart_title(self, node):
        print "DEPTIT", self.section_level,  node
        if self.section_level == 1:
            if self.section_title == '':
                start = self.body_len_before_node[node.__class__.__name__]
                self.section_title = ''.join(self.body[start + 1:])
                print "DEPART TITLE", self.section_title, node#,  self.body
        html4css1.HTMLTranslator.depart_title(self, node)

    def depart_author(self, node):
        start = self.body_len_before_node[node.__class__.__name__]
        self.authors.append(node.children[0])

    def visit_transition2(self, node):
        # hack to have titleless chapter
        self.reset_chapter()

    def visit_section(self, node):
        # too many divs is bad for mobi...
        if self.section_level == 0:
            if self.body:
                self.create_chapter()
            else:
                self.reset_chapter()
        self.section_level += 1
        self.first_paragraph = True

    def depart_section(self, node):
        self.first_page = False
        if self.section_level >= 1:
            self.section_level -= 1
        if self.section_level == 0:
            self.create_chapter()

    def visit_generated(self, node):
        print "GEN", node

    #depart_generated = depart_section

    def create_chapter(self):
        self.sections.append(self.body)
        self.body = []
        body = ''.join(self.sections[-1])
        if smartypants:
            #body = smartypants.smartyPants(body)
            # pass need to ignore pre contents...
            pass
        css = ''
        if self.css:
            css = ''.join(['<link rel="stylesheet" href="{0}" type="text/css" media="all" />'.format(os.path.basename(item)) for item in self.css])
            for item in self.css:
                if os.path.exists(item):
                    self.book.add_css(item, os.path.basename(item))
                else:
                    self.book.add_css(os.path.join(os.path.dirname(epub.__file__),
                                      'templates',
                                      'main.css'),'main.css')
        if self.font:
            for item in self.font:
                if os.path.exists(item):
                    self.book.add_font(item, os.path.basename(item))
                else:
                    raise KeyError
        js = ''
        if self.js:

            js = ''.join(['<script src="{0}" type="text/javascript"></script>'.format(os.path.basename(item)) for item in self.js])
            for item in self.js:
                if os.path.exists(item):
                    self.book.add_js(item, os.path.basename(item))
                else:
                    raise KeyError
        title = ''
        if 'title' in self.fields and self.is_title_page:
            title = self.fields['title']
        elif self.section_title:
            title=striptags(self.section_title)

        header = css+js
        html = XHTML_WRAPPER.format(body=body,
                                    title=title,
                                    header=header)
        if self.is_title_page:
            self.book.add_title_page(html)
            # clear out toc_map_node
            self.book.last_node_at_depth = {0:self.book.toc_map_root}

            self.is_title_page = False
        elif self.toc_page:
            self.book.add_toc_page(order=self.book.next_order())
            self.toc_page = False
        else:
            dst = '{0}.html'.format(len(self.sections))
            item = self.book.add_html('', dst, html)
            if self.guide_type:
                self.book.add_guide_item(dst, self.section_title, self.guide_type)
            self.book.add_spine_item(item)
            parent = self.toc_parents[-1] if self.toc_parents else None
            if self.toc_entry:
                node = self.book.add_toc_map_node(item.dest_path, striptags(self.section_title), parent=parent) #''.join(self.html_subtitle))
            if self.parent_level == 1:
                self.toc_parents = [node]
            elif self.parent_level == 2:
                self.toc_parents = self.toc_parents[:1] + [node]
        self.reset_chapter()

    def reset_chapter(self):
        self.section_title = ''
        self.first_paragraph = True
        self.css = ['main.css']
        self.font = []
        self.js = []
        self.parent_level = 0
        self.section_level = 0
        self.guide_type = None
        self.toc_entry = True  # page that has entry in TOC and NCX

    def visit_tgroup(self, node):
        # don't want colgroup
        node.stubs = []
        pass

    def visit_tbody(self, node):
        # don't want tbody
        pass

    def depart_tbody(self, node):
        pass

    visit_thead=visit_tbody
    depart_thead=depart_tbody

    def get_output(self):
        root_dir = '/tmp/epub'
        for k,v in self.fields.items():
            if k == 'creator':
                self.book.add_creator(v)
            elif k.lower() == 'title':
                self.book.set_title(v)
            else:
                self.book.add_meta(k, v)
        self.book.add_creator(', '.join(self.authors))
        if self.cover_image:
            self.book.add_cover(self.cover_image, title=''.join(self.title))
        for i, img_paths in enumerate(self.images.items()):
            abs_path, dst_path = img_paths
            self.book.add_image(abs_path, dst_path, id='image_{0}'.format(i))
        self.book.create_book(root_dir)
        self.book.create_archive(root_dir, root_dir + '.epub')
        return open(root_dir + '.epub').read()

XHTML_WRAPPER = u'''<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"
"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
<title>{title}</title>
<meta http-equiv="Content-type" content="application/xhtml+xml;charset=utf8" />
{header}
</head>
<body>
{body}
</body>
</html>'''


class EpubFileOutput(io.FileOutput):
    """
    A version of docutils.io.FileOutput which writes to a binary file.
    """
    def open(self):
        try:
            self.destination = open(self.destination_path, 'wb')

        except IOError, error:
            if not self.handle_io_errors:
                raise
            print >>sys.stderr, '%s: %s' % (error.__class__.__name__,
                                            error)
            print >>sys.stderr, ('Unable to open destination file for writing '
                                 '(%r).  Exiting.' % self.destination_path)
            sys.exit(1)
        self.opened = 1


class epubcontent(nodes.Element):
    # change normal TOC to epubcontent
    tagname = 'epubcontent'


class Index(Directive):
    """
    Directive to add entries to the index.
    """
    has_content = False
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    option_spec = {}
    count = 0

    def run(self):
        return [] #None #ignore for now
        # see sphinx.directives.other.Index for hints
        arguments = self.arguments[0].split('\n')
        targetid = 'index-%s' % Index.count
        Index.count += 1
        targetnode = nodes.target('', '', ids=[targetid])
        self.state.document.note_explicit_target(targetnode)
        indexnode = addnodes.index()
        indexnode['entries'] = ne = []
        indexnode['inline'] = False
        for entry in arguments:
            ne.extend(process_index_entry(entry, targetid))
        return [indexnode, targetnode]

class Contents(Directive):
    required_arguments = 0
    optional_arguments = 1
    final_argument_whitespace = True
    option_spec = { 'class': directives.class_option,
                    'local': directives.flag,
                    'depth': directives.nonnegative_int,
                    'page-numbers': directives.flag }
    def run(self):
        return [epubcontent()]


class Parser(docutils.parsers.rst.Parser):
    def __init__(self):
        directives.register_directive('contents', Contents)
        directives.register_directive('index', Index)
        docutils.parsers.rst.Parser.__init__(self)

# fix envvar !!!
from docutils import nodes
class envvar(nodes.Inline, nodes.TextElement): pass
def ignore_role(role, rawtext, text, lineno, inliner,
                       options={}, content=[]):
    return [envvar(rawtext, text)],[]

    #class envvar(Inline, TextElement):

#class envvar(nodes.Inline, nodes.TextElement): pass
#envvar = nodes.literal

from docutils.parsers.rst import roles
roles.register_local_role('envvar', ignore_role)
#roles.register_local_role('envvar', envvar)

def main(args):
    print "ARGS", args
    argv = None
    reader = standalone.Reader()
    reader_name = 'standalone'
    writer = EpubWriter()
    writer_name = 'epub2'
    parser = Parser()
    parser_name = 'restructuredtext'
    settings = None
    settings_spec = None
    settings_overrides = None
    config_section = None
    enable_exit_status = 1
    usage = default_usage
    publisher = Publisher(reader, parser, writer, settings,
                          destination_class=EpubFileOutput)
    publisher.set_components(reader_name, parser_name, writer_name)
    description = ('Generates epub books from reStructuredText sources.  ' + default_description)

    output = publisher.publish(argv, usage, description,
                               settings_spec, settings_overrides,
                               config_section=config_section,
                               enable_exit_status=enable_exit_status)

if __name__ == '__main__':
    if '--doctest' in sys.argv:
        _test()
    else:
        sys.exit(main(sys.argv) or 0)



import logging
import math

from genshi.template import MarkupTemplate, NewTextTemplate
from genshi.template import TemplateLoader
from genshi.core import Markup
from epublib import epub

logging.basicConfig(level=logging.DEBUG)
# see http://blog.epubandebookhelp.com/2012/05/16/kf8-panel-magnification/
# and amazon sample book
# http://mathorsinfotech.blogspot.com/2012/06/kf8-fixed-layout-image-zoom-tutorial.html
class MobiComicBook(epub.EpubBook):
    def __init__(self, width=600,
                 height=1024,
                 book_type='comic',
                 region_magnification='true',
                 orientation_lock='none',
                 add_jquery=False):
        epub.EpubBook.__init__(self)
        self.comic_meta = []
        self.add_comic_meta('fixed-layout', 'true')
        self.add_comic_meta('orientation-lock', 'portrait' )#orientation_lock)
        self.add_comic_meta('RegionMagnification', 'true')
        self.width = width
        self.height = height
        resolution = '{0}x{1}'.format(width, height)
        self.add_comic_meta('original-resolution', resolution)
        # if book type not specified images limited to 256K
        self.add_comic_meta('book-type', book_type)

        self.add_comic_meta('zero-gutter', 'true')
        self.add_comic_meta('zero-margin', 'true')


        self.pages = []
        self.page = None
        self.ordinal = 1
        self.target_id = 1

        self.add_jquery = add_jquery

    def get_target_id(self):
        tid =  'zoom{0}'.format(self.target_id)
        self.target_id += 1
        return tid

    def get_ordinal(self):
        # appears to be global for book
        self.ordinal += 1
        return self.ordinal - 1

    def get_meta_tags(self):
        tags = epub.EpubBook.get_meta_tags(self)
        value = end = ''
        for name, content in self.comic_meta:
            tags.append(('<meta name="{0}" content="{1}"/>'.format(name, content), value, end))
        return tags


    def add_comic_meta(self, name, content):
        self.comic_meta.append((name, content))

    def add_page(self):
        page = Page(self)
        self.pages.append(page)
        return page



class Mag(object):
    def __init__(self, txt, target_id_parent, target_id,
                 left, top, width, height,
                 lb_left, lb_top, lb_width, lb_height,
                 zoomed_img_left, zoomed_img_top,
                 img_src, ordinal=None, zoom_factor=1.5,
                 pre_data=None, post_data=None):
        self.txt = txt
        self.target_id_parent = target_id_parent
        self.target_id = target_id

        # coordinates for click target
        self.top = top
        self.left = left
        self.height = height
        self.width = width

        # lb_ refers to lightbox image
        self.lb_top = lb_top
        self.lb_left = lb_left
        self.lb_width = lb_width
        self.lb_height = lb_height

        # zoomed_img_ referes to position on zoomed image relative to lightbox
        self.zoomed_img_top = zoomed_img_top
        self.zoomed_img_left = zoomed_img_left

        self.ordinal = ordinal
        self.zoom_factor = zoom_factor
        self.img_src = img_src
        #text/etc to display
        if post_data:
            self.post_data = Markup(post_data)
        else:
            self.post_data = ''
        if pre_data:
            self.pre_data = Markup(pre_data)
        else:
            self.pre_data = ''

    @property
    def target_id_mag(self):
        return self.target_id + "-magTarget"


    def json_junk(self):
        # kf8 really cares about double quotes on the inside...
        # only took 33 attempts to figure this out
        result = '{{"targetId":"{0}", "ordinal":{1} }}'.format(self.target_id_parent, self.ordinal)
        return result

LEFT=1
RIGHT=2
DOWN=3
UP=4

class Page(object):
    PARENT_SUFFIX = '-magTargetParent'
    def __init__(self, book, height=1024, width=600):
        self.img = None
        self.book = book
        self.mags = []
        self.title = None
        self.style = None
        self.height = height
        self.width = width


    def __iter__(self):
        for mag in self.mags:
            yield mag

    def add_bg_image(self, src_path, dest_path, id=None):
        self.img = self.book.add_image(src_path, dest_path, id)

    def auto_target(self,
                    whole_target_left, whole_target_top, whole_target_width, whole_target_height,
                    zoom_factor=2, overlap_percent=10, direction=LEFT,
                    post_data=None,
                    txt=None):
        start_left = whole_target_left
        # target is all in percent
        print "ZF", zoom_factor
        if direction == LEFT:
            chunks = whole_target_width*zoom_factor/100.
            chunk_int = int(math.ceil(chunks))
            target_left = whole_target_left
            zoom_left = whole_target_left
            print "CHUNKS", chunks, "WHOLE WIDTH", whole_target_width
            for i in range(chunk_int):
                if chunks > 1:
                    target_width = whole_target_width/float(chunks)
                else:
                    target_width = whole_target_width
                right_side = target_left + target_width
                if right_side > 100: #whole_target_width:
                    #target_width = right_side - target_left
                    target_left = 100 - target_width
                    zoom_left = 100 - target_width
                elif right_side > start_left + whole_target_width:
                    print "OFF RIGHT",
                    target_left = whole_target_left + whole_target_width - target_width
                    zoom_left = whole_target_left + whole_target_width - target_width
                    #target_width = whole_target_width # - target_left
                print "LEFT", target_left
                print "RIGHT SIDE", right_side, "TW", target_width

                print "H", self.book.height, "MINUS", - whole_target_height*zoom_factor/2.
                lb_width = 100
                lb_left = 0
                if whole_target_width * zoom_factor < 100:
                    print "SMALLER", whole_target_width * zoom_factor
                    lb_width = whole_target_width * zoom_factor
                    lb_left = 50 - lb_width/2
                self.add_zoom_image(zoom_factor,
                    target_left, whole_target_top, target_width, whole_target_height,
                    lb_left, 50. - whole_target_height*zoom_factor/2., lb_width, whole_target_height*zoom_factor,
                    -zoom_left, 0 - 100*(whole_target_top/float(whole_target_height)/zoom_factor),
                    txt=txt,post_data=post_data)
                #zoom_left, -whole_target_top*zoom_factor)

                zoom_left += target_width
                target_left += whole_target_width/float(chunks)
                #target_left += (whole_target_width - target_width)/float(chunks)

    def add_zoom_image(self, zoom_factor,
                       target_left, target_top, target_width, target_height,
                       lb_left, lb_top, lb_width, lb_height,
                       zoom_left, zoom_top,
                       src_path=None, dest_path=None, txt=None,
                       pre_data=None, post_data=None):
        # post data is borked
        logging.debug("ZOOM IMAGE: tl:{0} tt:{1}: tw:{2} th:{3}\n lb_l:{4} lb_t:{5} lb_w:{6}, lb_h:{7}\n  zl:{8} zt:{9}".format(
            target_left, target_top, target_width, target_height,
              lb_left, lb_top, lb_width, lb_height,
              zoom_left, zoom_top
            ))
        print "BAR"
        if src_path and dest_path:
            img = self.book.add_image(src_path, dest_path, None)
        else:
            img = self.img
        tid = self.book.get_target_id()
        ordinal = self.book.get_ordinal()
        mag = Mag(txt, tid+self.PARENT_SUFFIX, tid,
            int(target_left), int(target_top), int(target_width), int(target_height),
            int(lb_left), int(lb_top), int(lb_width), int(lb_height),
            int(zoom_left), int(zoom_top),
            img.dest_path, ordinal,
            pre_data=pre_data, post_data=post_data,
            zoom_factor=zoom_factor)
        self.mags.append(mag)

    def to_html(self):
        #temp = NewTextTemplate(PAGE_TEMPLATE)
        temp = self.book.loader.load('mobicomic2.html')
        stream = temp.generate(page=self, body='', img_src=self.img.dest_path,
                               add_jquery=self.book.add_jquery)
        html = stream.render('html', doctype= 'html-transitional')
        return html

    def add_html(self, dest_path):
        return self.book.add_html('', dest_path, self.to_html())


TEST_HTML = """
        <!DOCTYPE html SYSTEM "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">

<head>
	<title>KF8 SAMPLE TEMPLATE</title>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
  <style>
  /*
Copyright (c) 2010, Yahoo! Inc. All rights reserved.
Code licensed under the BSD License:
http://developer.yahoo.com/yui/license.html
version: 3.3.0
build: 3167
*/
html{color:#000;background:#FFF;}body,div,dl,dt,dd,ul,ol,li,h1,h2,h3,h4,h5,h6,pre,code,form,fieldset,legend,input,textarea,p,blockquote,th,td{margin:0;padding:0;}table{border-collapse:collapse;border-spacing:0;}fieldset,img{border:0;}address,caption,cite,code,dfn,em,strong,th,var{font-style:normal;font-weight:normal;}li{list-style:none;}caption,th{text-align:left;}h1,h2,h3,h4,h5,h6{font-size:100%;font-weight:normal;}q:before,q:after{content:'';}abbr,acronym{border:0;font-variant:normal;}sup{vertical-align:text-top;}sub{vertical-align:text-bottom;}input,textarea,select{font-family:inherit;font-size:inherit;font-weight:inherit;}input,textarea,select{*font-size:100%;}legend{color:#000;}

@font-face {
	font-family: "Booter 5-0";
	font-style: normal;
	font-weight: normal;
	src: url(../fonts/Booter_5-0.ttf);
}

body {
	font-family: "Booter 5-0";
	font-size: 210%;
}

div.fs {
  height: 1024px;
  width: 600px;
  position: relative;
}

div.fs div {
  position: absolute;
}

div.fs a {
  display: block;
  width : 100%;
  height: 100%;
}

.image {
  position: absolute;
  height: 1024px;
  width: 600px;
}


div.note {
	top: 10%;
	left: 0%;
	width: 100%;
}

div.center {
	position: absolute;
	top: 42%;
	left: 29%;
	width: 40%;
  border: 4px solid #000000;
}

h1 {	font-family: "Booter 5-0";
	font-size: 45px;
	position: relative;
	text-align: center;
	text-shadow: 1px 1px 3px #1a1a1a;
	top: 0px;
}
h2 {	font-family: "Booter 5-0";
	font-size: 32px;
	position: relative;
	text-align: center;
	font-style: italic;
	text-shadow: 1px 1px 3px #1a1a1a;
	top: 30px;
}

p {
	font-family: "Booter 5-0";
	font-style: italic;
	position: relative;
	text-align: center;
	text-shadow: 1px 1px 3px #1a1a1a;
}

p.textzoom {
	font-size: 150%;
	margin: 20%;
	top: 18%;
	z-index: 1;
}

div.target-mag {
  position: absolute;
  display: block;
  overflow: hidden;
  /* UI Styling */
  border: 5px solid  #000000;
}

/* Actual image resolution */
div.target-mag img {
  position: absolute;

  height: 3000px;
  /*height: 1536px;
    width: 900px;*/
  width: 1800px;
}
div.target-mag-parent {
	width:100%;
	height:100%;
	display:none;
}
div.target-mag-lb {
	height: 100%;
	width: 100%;
	background: #333333;
	opacity:.75;
}

/* Use the following to define the zoom levels */
/* zoom100 equals the default display size */

div.target-mag img.zoom100 {
  position: absolute;
  height: 1024px;
  width: 600px;
}

/* Enter the actual height and width of the image here: this is the zoom factor, regardless of what number you enter in the tag at zoom### - that number only need be larger than 100, but otherwise has no effect on the amount of magnification */

div.target-mag img.zoom150 {
  position: absolute;
  height: 1536px;
  width: 900px;
}

/* Tap Target Zone Size & Position */
#zoom1 {
	top: 30%;
	left: 15%;
	height: 45%;
	width: 70%;
}
/* Magnified Region Size & Position */
#zoom1-magTarget {
	top: 21%;
	left: 0%;
	height: 62%;
	width: 100%;
}
/* Image Offset */
#zoom1-magTarget img {
	top: -77%;
	left: -25%;
}


/* Magnified text box formatting */

/* Default content */

p.text {
	font-size: 100%;
	border: 4px solid #000000;
	padding: 10px;
}

.app-amzn-magnify {
 background-color: green;
border: 5px solid  #000000;
opacity: .5;
}


</style>
</head>

<body>

<!-- This is an example of how to use region zoom to magnify an image -->
	<div class="fs">
		<div>
			<img src="img3.jpeg" alt="The Serpent Ring" class="image"/>
		</div>

		<div class="note">
			<h1>Example of Panel View<br/>
			with Lightbox effect</h1>
			<h2>showing how to use live text<br/>
			within a magnified image</h2>
		</div>


     <div id="zoom1">
<!--			<a class="app-amzn-magnify" data-app-amzn-magnify="{'targetId':'zoom1-magTargetParent', 'ordinal':4}" works='False'> -->
			<a class="app-amzn-magnify" data-app-amzn-magnify='{"targetId":"zoom1-magTargetParent", "ordinal":4}' works='True'>
              <div><p class="center">BEFORE</p></div>
            </a>
      </div>

      <div id="zoom1-magTargetParent" class="target-mag-parent">
        <div class="target-mag-lb"></div>
        <div id="zoom1-magTarget" class="target-mag">
          <h2>AFTER</h2>
          <img src="img1.jpeg" alt="">
        </div>
      </div>


	</div>

</body>
</html>

        """

def test():
    book = MobiComicBook(add_jquery=True)
    title = 'Test Comic35'
    book.set_title(title)
    cov_image = 'data/little-nemo-19051015-l.jpeg'
    book.add_cover(cov_image, title=title)
    page = book.add_page()
    page.add_bg_image('data/little-nemo-19051015-l.jpeg', 'img1.jpeg')
    #page.add_mag('foo_id_parent', 'foo_id', 20, 20, 30, 60, 30, 30)
    page.auto_target(50, 1*100./6, 50, 100./6, #target_left, target_top, target_width, target_height,
                    zoom_factor=3, post_data='<p class="textzoom">TEST DATA</p>')
    # page.auto_target(0, 0, 100, 100./6, #target_left, target_top, target_width, target_height,
    #                 zoom_factor=2)
    # for row in range(1,5):
    #     page.auto_target(0, row*100./6, 50, 100./6, #target_left, target_top, target_width, target_height,
    #                 zoom_factor=2)
    #     page.auto_target(50, row*100./6, 50, 100./6, #target_left, target_top, target_width, target_height,
    #                 zoom_factor=2)
    for col in range(4):
        page.auto_target(col*25, 500./6, 25, 100./6, #target_left, target_top, target_width, target_height,
                    zoom_factor=2)

        #page.add_zoom_image(1.5,50,0,100./6,50,50,0,100,20,0,50,pre_data='<div><p class="center">BEFORE</p></div>')
    h1 = page.add_html('1.html')
    book.add_spine_item(h1)

    # page = book.add_page()
    # page.add_bg_image('data/little-nemo-19051022-l.jpeg', 'img2.jpeg')
    # page.add_zoom_image(1.5, 0,0,50,50,0,0,pre_data='<p>BEFORE</p>',
    #                     post_data='<h2>AFTER</h2>')
    # h2 = page.add_html('2.html')
    # book.add_spine_item(h2)

    page.add_bg_image('data/little-nemo-19051105-l.jpeg', 'img3.jpeg')
    h3 = book.add_html('', '3.html', TEST_HTML)
    book.add_spine_item(h3)
    directory = '/tmp/mobitest'
    book.create_book(directory)
    book.create_archive(directory, directory + '.epub')


if __name__ == '__main__':
    test()



try:
    from setuptools import setup
except:
    from distutils.core import setup

setup(name="rst2epub2",
      version='0.2',
      author='matt harrison',
      author_email='matthewharrison AT gmail',
      description="script/library to create epubs programatically and from rst",
      long_description=open('README.rst').read(),
      url='https://github.com/mattharrison/rst2epub2',
      install_requires=['docutils', 'genshi'],
      scripts=["rst2epub.py"],
      package_dir={"epublib":"epublib"},
      packages=['epublib'],
      package_data={'epublib':['templates/*.css','templates/*.html','templates/*.ncx','templates/*.xml', 'templates/*.opf']},
      zip_safe=False,
      classifiers=(
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',)
)



# -*- coding: utf-8 -*-
"""
Copyright (c) 2011, timtambin@gmail.com
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
    Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
import itertools
import mimetypes
import os
import shutil
import subprocess
import uuid
import zipfile


from genshi.template import TemplateLoader
try:
    from lxml import etree
except ImportError as e:
    # no xpath support!
    import xml.etree.ElementTree as etree


COVER_ORDER = -300
TITLE_ORDER = -200
TOC_ORDER = -100

class TocMapNode:

    def __init__(self):
        self.play_order = 0
        self.title = ''
        self.href = ''
        self.children = []
        self.depth = 0

    def assign_play_order(self):
        next_play_order = [0]
        self._assign_play_order(next_play_order)

    def _assign_play_order(self, next_play_order):
        self.play_order = next_play_order[0]
        next_play_order[0] = self.play_order + 1
        for child in self.children:
            child._assign_play_order(next_play_order)


class EpubItem:

    def __init__(self):
        self.id = ''
        self.src_path = ''
        self.dest_path = ''
        self.mime_type = ''
        self.html = ''


class EpubBook:

    def __init__(self):
        #self.loader = TemplateLoader('epublib/templates')
        temp_dir = os.path.dirname(os.path.abspath(__file__))
        self.loader = TemplateLoader(os.path.join(temp_dir,'templates'))

        self.root_dir = ''
        self.UUID = uuid.uuid1()

        self.lang = 'en-US'
        self.title = ''
        self.creators = []
        self.meta_info = []

        self.image_items = {}
        self.html_items = {}
        self.css_items = {}
        self.js_items = {}
        self.font_items = {}


        self.cover_image = None
        self.title_page = None
        self.toc_page = None

        self.spine = []
        self.guide = {}
        self.toc_map_root = TocMapNode()
        self.last_node_at_depth = {0 : self.toc_map_root}


    def set_title(self, title):
        self.title = title

    def set_lang(self, lang):
        self.lang = lang

    def add_creator(self, name, role = 'aut'):
        self.creators.append((name, role))

    def add_meta(self, meta_name, meta_value, **meta_attrs):
        self.meta_info.append((meta_name, meta_value, meta_attrs))

    def get_meta_tags(self):
        l = []
        for meta_name, meta_value, meta_attr in self.meta_info:
            begin_tag = '<dc:%s' % meta_name
            if meta_attr:
                for attr_name, attr_value in meta_attr.iteritems():
                    begin_tag += ' opf:%s="%s"' % (attr_name, attr_value)
            begin_tag += '>'
            end_tag = '</dc:%s>' % meta_name
            l.append((begin_tag, meta_value, end_tag))
        return l

    def get_image_items(self):
        return sorted(self.image_items.values(), key = lambda x : x.id)

    def get_html_items(self):
        return sorted(self.html_items.values(), key = lambda x : x.id)

    def get_css_items(self):
        return sorted(self.css_items.values(), key = lambda x : x.id)

    def get_js_items(self):
        return sorted(self.js_items.values(), key = lambda x : x.id)

    def get_all_items(self):
        return sorted(itertools.chain(self.image_items.values(), self.html_items.values(), self.css_items.values(), self.js_items.values(), self.font_items.values()), key = lambda x : x.id)

    def add_image(self, src_path, dest_path, id=None):
        if dest_path in self.image_items:
            return
        item = EpubItem()
        item.id = id or 'image_{0}'.format(len(self.image_items) + 1)
        item.src_path = src_path
        item.dest_path = dest_path
        item.mime_type = mimetypes.guess_type(dest_path)[0]
        #assert item.dest_path not in self.image_items
        self.image_items[dest_path] = item
        return item


    def add_html(self, src_path, dest_path, html, id=None):
        item = EpubItem()
        item.id = id or 'html_%04d' % (len(self.html_items) + 1)
        item.src_path = src_path
        item.dest_path = dest_path
        item.html = html
        item.mime_type = 'application/xhtml+xml'
        assert item.dest_path not in self.html_items
        self.html_items[item.dest_path] = item
        return item

    def add_font(self, src_path, dest_path):
        if dest_path in self.font_items:
            return
        item = EpubItem()
        item.id = 'font_%d' % (len(self.font_items) + 1)
        item.src_path = src_path
        item.dest_path = dest_path
        if src_path.endswith('otf'):
            item.mime_type = 'application/opentype'
        elif src_path.endswith('ttf'):
            item.mime_type = 'application/truetype'
        else:
            raise KeyError
        #assert item.dest_path not in self.font_items
        self.font_items[item.dest_path] = item
        return item

    def add_js(self, src_path, dest_path):
        if dest_path in self.js_items:
            return
        item = EpubItem()
        item.id = 'js_%d' % (len(self.css_items) + 1)
        item.src_path = src_path
        item.dest_path = dest_path
        item.mime_type = 'text/javascript'
        #assert item.dest_path not in self.css_items
        self.js_items[item.dest_path] = item
        return item

    def add_css(self, src_path, dest_path):
        if dest_path in self.css_items:
            return
        item = EpubItem()
        item.id = 'css_%d' % (len(self.css_items) + 1)
        item.src_path = src_path
        item.dest_path = dest_path
        item.mime_type = 'text/css'
        #assert item.dest_path not in self.css_items
        self.css_items[item.dest_path] = item
        return item

    def add_cover(self, src_path, title='_cover'):
        assert not self.cover_image
        _, ext = os.path.splitext(src_path)
        dest_path = 'cover%s' % ext
        self.cover_image = self.add_image(src_path, dest_path, id='cover-image')
        cover_page = self.add_cover_html_for_image(self.cover_image, title)
        self.add_spine_item(cover_page, False, COVER_ORDER)
        self.add_guide_item(cover_page.dest_path, title, 'cover')

    def add_cover_html_for_image(self, image_item, title):
        tmpl = self.loader.load('image.html')
        image_item.title = title
        stream = tmpl.generate(book=self, item=image_item)
        html = stream.render('xhtml', doctype='xhtml11', drop_xml_decl=False)
        return self.add_html('', 'cover.html', html, id='cover')

    def _make_title_page(self):
        assert self.title_page
        if self.title_page.html:
            return
        tmpl = self.loader.load('title-page.html')
        stream = tmpl.generate(book = self)
        self.title_page.html = stream.render('xhtml', doctype = 'xhtml11', drop_xml_decl = False)

    def add_title_page(self, html=''):
        assert not self.title_page
        self.title_page = self.add_html('', 'title-page.html', html)
        self.add_spine_item(self.title_page, True, TITLE_ORDER)
        self.add_guide_item('title-page.html', 'Title Page', 'title-page')

    def _make_toc_page(self):
        assert self.toc_page
        tmpl = self.loader.load('toc.html')
        stream = tmpl.generate(book=self)
        self.toc_page.html = stream.render('xhtml', doctype = 'xhtml11', drop_xml_decl = False)

    def add_toc_page(self, order=TOC_ORDER):
        assert not self.toc_page
        self.toc_page = self.add_html('', 'toc.html', '')
        self.add_spine_item(self.toc_page, False, order)
        self.add_guide_item('toc.html', 'Table of Contents', 'toc')

    def get_spine(self):
        results = sorted(self.spine)
        return results

    def next_order(self):
        order = (max(order for order, _, _ in self.spine) if self.spine else 0) + 1
        return order

    def add_spine_item(self, item, linear=True, order=None):
        assert item.dest_path in self.html_items
        if order is None:
            order = self.next_order()
        self.spine.append((order, item, linear))

    def get_guide(self):
        return sorted(self.guide.values(), key = lambda x : x[2])

    def add_guide_item(self, href, title, type):
        assert type not in self.guide, "TYPE %s NOT IN GUIDE %s"%(type, self.guide)
        self.guide[type] = (href, title, type)

    def get_toc_map_root(self):
        return self.toc_map_root

    def get_toc_map_height(self):
        return max(self.last_node_at_depth.keys())

    def add_toc_map_node(self, href, title, depth=None, parent=None):
        node = TocMapNode()
        node.href = href
        node.title = title
        if parent == None:
            if depth == None:
                parent = self.toc_map_root
            else:
                parent = self.last_node_at_depth[depth - 1]
        parent.children.append(node)
        node.depth = parent.depth + 1
        self.last_node_at_depth[node.depth] = node
        return node

    def make_dirs(self):
        try:
            os.makedirs(os.path.join(self.root_dir, 'META-INF'))
        except OSError:
            pass
        try:
            os.makedirs(os.path.join(self.root_dir, 'OEBPS'))
        except OSError:
            pass

    def _write_container_xml(self):
        fout = open(os.path.join(self.root_dir, 'META-INF', 'container.xml'), 'w')
        tmpl = self.loader.load('container.xml')
        stream = tmpl.generate()
        fout.write(stream.render('xml'))
        fout.close()

    def _write_toc_ncx(self):
        self.toc_map_root.assign_play_order()
        fout = open(os.path.join(self.root_dir, 'OEBPS', 'toc.ncx'), 'w')
        tmpl = self.loader.load('toc.ncx')
        stream = tmpl.generate(book=self)
        fout.write(stream.render('xml'))
        fout.close()

    def _write_content_opf(self):
        fout = open(os.path.join(self.root_dir, 'OEBPS', 'content.opf'), 'w')
        tmpl = self.loader.load('content.opf')
        stream = tmpl.generate(book=self)
        fout.write(stream.render('xml'))
        fout.close()

    def _write_items(self):
        for item in self.get_all_items():
            if item.html:
                fout = open(os.path.join(self.root_dir, 'OEBPS', item.dest_path), 'w')
                if isinstance(item.html, str):
                    fout.write(item.html)
                else:
                    fout.write(item.html.encode('utf-8'))
                #fout.write(item.html.decode('utf-8'))
                #fout.write(item.html)
                fout.close()
            else:
                #shutil.copyfile(item.src_path, os.path.join(self.root_dir, 'OEBPS', item.dest_path))
                put_file(item.src_path, os.path.join(self.root_dir, 'OEBPS', item.dest_path))


    def _write_mime_type(self):
        fout = open(os.path.join(self.root_dir, 'mimetype'), 'w')
        fout.write('application/epub+zip')
        fout.close()

    @staticmethod
    def _list_manifest_items(content_opf_path):
        tree = etree.parse(content_opf_path)
        #return tree.xpath("//opf:manifest/opf:item/@href", namespaces = {'opf': 'http://www.idpf.org/2007/opf'})
        return [foo.attrib['href'] for foo in tree.findall('{http://www.idpf.org/2007/opf}manifest/{http://www.idpf.org/2007/opf}item')]

    @staticmethod
    def create_archive(root_dir, output_path):
        fout = zipfile.ZipFile(output_path, 'w')
        cwd = os.getcwd()
        os.chdir(root_dir)
        fout.writestr('mimetype', 'application/epub+zip',
                      compress_type=zipfile.ZIP_STORED)
        fileList = []
        fileList.append(os.path.join('META-INF', 'container.xml'))
        fileList.append(os.path.join('OEBPS', 'content.opf'))
        for item_path in EpubBook._list_manifest_items(os.path.join('OEBPS', 'content.opf')):
            fileList.append(os.path.join('OEBPS', item_path))
        for file_path in fileList:
            fout.write(file_path, compress_type=zipfile.ZIP_DEFLATED)
        fout.close()
        os.chdir(cwd)

    @staticmethod
    def check_epub(checker_path, epub_path):
        subprocess.call(['java', '-jar', checker_path, epub_path], shell = True)

    def create_book(self, root_dir):
        if self.title_page:
            self._make_title_page()
        if self.toc_page:
            self._make_toc_page()
        self.root_dir = root_dir
        self.make_dirs()
        self._write_mime_type()
        self._write_items()
        self._write_container_xml()
        self._write_content_opf()
        self._write_toc_ncx()


def put_file(abs_path, rel_path):
    """
    given a file put it in the rel_path creating necessary dirs
    """
    parents = os.path.dirname(rel_path)
    try:
        os.makedirs(parents)

    except OSError, e:
        import errno
        if e.errno != errno.EEXIST or not os.path.isdir(parents):
            raise
    shutil.copyfile(abs_path, rel_path)

def test():
    def get_minimal_html(text):
        return """<!DOCTYPE html PUBLIC "-//W3C//DTD XHtml 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head><title>%s</title></head>
<body><p>%s</p></body>
</html>
""" % (text, text)

    book = EpubBook()
    book.set_title('Most Wanted Tips for Aspiring Young Pirates')
    book.add_creator('Monkey D Luffy')
    book.add_creator('Guybrush Threepwood')
    book.add_meta('contributor', 'Smalltalk80', role = 'bkp')
    book.add_meta('date', '2010', event = 'publication')

    book.add_title_page()
    book.add_toc_page()
    book.add_cover(r'D:\epub\blank.png')

    book.add_css(r'main.css', 'main.css')

    n1 = book.add_html('', '1.html', get_minimal_html('Chapter 1'))
    n11 = book.add_html('', '2.html', get_minimal_html('Section 1.1'))
    n111 = book.add_html('', '3.html', get_minimal_html('Subsection 1.1.1'))
    n12 = book.add_html('', '4.html', get_minimal_html('Section 1.2'))
    n2 = book.add_html('', '5.html', get_minimal_html('Chapter 2'))

    book.add_spine_item(n1)
    book.add_spine_item(n11)
    book.add_spine_item(n111)
    book.add_spine_item(n12)
    book.add_spine_item(n2)

    # You can use both forms to add TOC map
    #t1 = book.add_toc_map_node(n1.dest_path, '1')
    #t11 = book.add_toc_map_node(n11.dest_path, '1.1', parent = t1)
    #t111 = book.add_toc_map_node(n111.dest_path, '1.1.1', parent = t11)
    #t12 = book.add_toc_map_node(n12.dest_path, '1.2', parent = t1)
    #t2 = book.add_toc_map_node(n2.dest_path, '2')

    book.add_toc_map_node(n1.dest_path, '1')
    book.add_toc_map_node(n11.dest_path, '1.1', 2)
    book.add_toc_map_node(n111.dest_path, '1.1.1', 3)
    book.add_toc_map_node(n12.dest_path, '1.2', 2)
    book.add_toc_map_node(n2.dest_path, '2')

    root_dir = r'd:\epub\test'
    book.create_book(root_dir)
    EpubBook.create_archive(root_dir, root_dir + '.epub')
    EpubBook.check_epub('epubcheck-1.0.5.jar', root_dir + '.epub')

if __name__ == '__main__':
    test()
